#include "wifi_check.h"
#include "ets_sys.h"
#include "osapi.h"
#include "user_interface.h"

typedef enum {
    WIFI_CONNECTING,
    WIFI_CONNECTING_ERROR,
    WIFI_CONNECTED,
} tConnState;

enum { REFRESH_TIME = 15000 };

static struct ip_info ipConfig;
static ETSTimer WiFiLinker;
static tConnState connState = WIFI_CONNECTING;

struct netif * eagle_lwip_getif(uint8 index);

static int ICACHE_FLASH_ATTR igmp_join(uint32_t ip) {
    if( ip ) {
        ip_addr_t ipgroup;
        int ret;
        ipaddr_aton(MULTICAST_ADDR, &ipgroup);
        //os_printf( "IGMP Joining: %08x %08x\n", ip, ipgroup.addr );
        ret = espconn_igmp_join( (ip_addr_t *)&ip, &ipgroup);
        if (ret != 0) {
            os_printf("IGMP join failed, code %d", ret);
        }
        return 0;
    }
    os_printf("Failed to obtain current IP\n");
    return 1;
}

const char *WiFiStatus[] = {
    "STATION_IDLE", 			// 0x00
    "STATION_CONNECTING", 		// 0x01
    "STATION_WRONG_PASSWORD", 	// 0x02
    "STATION_NO_AP_FOUND", 		// 0x03
    "STATION_CONNECT_FAIL", 	// 0x04
    "STATION_GOT_IP" 			// 0x05
};

static void ICACHE_FLASH_ATTR wifi_check_ip(void *arg) {
    os_timer_disarm(&WiFiLinker);
    switch(wifi_station_get_connect_status()) {
    case STATION_GOT_IP:
        wifi_get_ip_info(STATION_IF, &ipConfig);
        if(ipConfig.ip.addr != 0) {
            connState = WIFI_CONNECTED;
            os_printf("WiFi is connected\r\n");
            //TODO: IGMP not working (?)
            //igmp_join(ipConfig.ip.addr);
        } else {
            connState = WIFI_CONNECTING_ERROR;
            os_printf("WiFi connected, ip.addr is null\r\n");
        }
        break;
    case STATION_WRONG_PASSWORD:
        connState = WIFI_CONNECTING_ERROR;
        os_printf("WiFi connecting error, wrong password\r\n");
        break;
    case STATION_NO_AP_FOUND:
        connState = WIFI_CONNECTING_ERROR;
        os_printf("WiFi connecting error, ap not found\r\n");
        break;
    case STATION_CONNECT_FAIL:
        connState = WIFI_CONNECTING_ERROR;
        os_printf("WiFi connecting fail\r\n");
        break;
    default:
        connState = WIFI_CONNECTING;
        os_printf("WiFi connecting...\r\n");
    }
    os_timer_setfn(&WiFiLinker, (os_timer_func_t *)wifi_check_ip, NULL);
    os_timer_arm(&WiFiLinker, REFRESH_TIME, 0);
}

void wifi_check_init(void) {
    // Wait for Wi-Fi connection
    os_timer_disarm(&WiFiLinker);
    os_timer_setfn(&WiFiLinker, (os_timer_func_t *)wifi_check_ip, NULL);
    os_timer_arm(&WiFiLinker, REFRESH_TIME, 0);
}
